@@ -1,5 +1,4 @@
 import datetime
-import functools
 import importlib
 import inspect
 import logging
@@ -9,16 +8,21 @@ import traceback
 from dataclasses import dataclass
 from pathlib import Path
 from typing import TYPE_CHECKING, Any, List
+from uuid import uuid4
 
 import anyio
 import schedule
 from anyio import create_task_group
 from dataclasses_json import dataclass_json
 
+from databutton.decorators.jobs.logger import JobRunLogger
+from databutton.decorators.jobs.utils import iso_utc_timestamp
+from databutton.utils import is_running_locally
+
 if TYPE_CHECKING:
     from databutton.utils.build import ArtifactDict
 
-logger = logging.getLogger("dattabutton.schedule")
+logger = logging.getLogger("databutton.schedule")
 
 
 @dataclass
@@ -84,7 +88,6 @@ class JobRun:
 class Job(schedule.Job):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        self.runs: List[JobRun] = []
         self.databutton_schedule: DatabuttonSchedule = None
 
     def from_databutton_schedule(self, databutton_schedule: DatabuttonSchedule):
@@ -98,81 +101,66 @@ class Job(schedule.Job):
         return self.do(func)
 
     async def run(self):
+        run_id = str(uuid4())
         if self.databutton_schedule is None:
-            logger.debug("Running job without a databutton schedule, what?")
+            job_id = None  # Raising instead because we need a job id below here!
+        else:
+            job_id = self.databutton_schedule.uid
+
+        if job_id is None and not is_running_locally():
+            raise Exception(
+                "Running job without a databutton schedule, need a job id to continue"
+            )
 
         if self._is_overdue(datetime.datetime.now()):
             logger.debug("Cancelling job %s", self)
             return schedule.CancelJob
 
-        try:
-            if inspect.iscoroutinefunction(self.job_func):
-                ret = await self.job_func()
-            else:
-                ret = self.job_func()
-        except Exception as e:
-
-            logger.error(traceback.format_exc())
-
-            if os.environ.get("SENTRY_DSN"):
-                import sentry_sdk
-
-                with sentry_sdk.push_scope() as scope:
-                    scope.set_tag("job.uid", self.databutton_schedule.uid)
-                    scope.set_extra("job.last_run", self.last_run)
-                    scope.set_extra("job.next_run", self.next_run)
-                    scope.set_extra("job.nruns", len(self.runs))
-                    sentry_sdk.capture_exception(e)
-
-            if self.databutton_schedule.cancel_on_failure:
-                ret = schedule.CancelJob
-            else:
-                ret = e  # Or None? Depends on how we use the jobrun history later
-
-        self.last_run = datetime.datetime.now()
-
-        # Only store the last 10 runs for memory reasons
-        self.runs.append(JobRun(timestamp=self.last_run, result=ret))
-        self.runs = self.runs[-10:]
-
-        self._schedule_next_run()
-
-        if self._is_overdue(datetime.datetime.now()):
-            logger.debug("Cancelling job %s", self)
-            return schedule.CancelJob
-        return ret
-
-
-def report_exceptions(cancel_on_failure=False, job: Job = None):
-    def catch_exceptions_decorator(job_func):
-        @functools.wraps(job_func)
-        async def wrapper(*args, **kwargs):
+        with JobRunLogger(logger, job_id=job_id, run_id=run_id) as jobrun:
             try:
-                ret = await job_func(*args, **kwargs)
-                return ret
+                if inspect.iscoroutinefunction(self.job_func):
+                    ret = await self.job_func()
+                else:
+                    ret = self.job_func()
+                jobrun.success = True
             except Exception as e:
-                import traceback
+                jobrun.success = False
 
-                logger.error(traceback.format_exc())
+                # This needs to be printed, as the stdout is handled by JobRunner
+                # Otherwise we would need to configure the thing to do standard out
+                print(traceback.format_exc())
 
                 if os.environ.get("SENTRY_DSN"):
                     import sentry_sdk
 
                     with sentry_sdk.push_scope() as scope:
-                        scope.set_tag("job.uid", job.uid)
-                        scope.set_extra("job.last_run", job.last_run)
-                        scope.set_extra("job.next_run", job.next_run)
-                        scope.set_extra("job.nruns", len(job.runs))
+                        scope.set_tag("job.uid", job_id)
+                        scope.set_tag("job.run_id", run_id)
+                        # Not bothering with fixing this now but I think these are wrong:
+                        # This will be when the previous run started?
+                        scope.set_extra("job.last_run", self.last_run)
+                        # This will be when this run was supposed to start?
+                        scope.set_extra("job.next_run", self.next_run)
                         sentry_sdk.capture_exception(e)
 
-                if cancel_on_failure:
-                    return schedule.CancelJob
+                if self.databutton_schedule.cancel_on_failure:
+                    ret = schedule.CancelJob
                 else:
-                    return None  # Just being explicit
+                    ret = e  # Or None? Depends on how we use the jobrun history later
 
-        return wrapper
+            self.last_run = jobrun.start_time
 
-    return catch_exceptions_decorator
+            self._schedule_next_run()
+
+            if self._is_overdue(datetime.datetime.now()):
+                logger.debug("Cancelling job %s", self)
+                return schedule.CancelJob
+
+            # Used on jobrun exit, not set if the cancel above hits
+            # (but I'm not sure I follow the original logic here)
+            jobrun.next_run_time = iso_utc_timestamp(self.next_run)
+
+        return ret
 
 
 class Scheduler(schedule.Scheduler):
