@@ -97,6 +97,18 @@ class CouldNotStartError(Exception):
     pass
 
 
+_streamlit_args = [
+    ("server.address", "0.0.0.0"),
+    ("server.enableWebsocketCompression", "true"),
+    ("browser.gatherUsageStats", "false"),
+    ("global.dataFrameSerialization", "arrow"),
+    ("server.headless", "true"),
+    ("server.enableCORS", "false"),
+    ("server.enableXsrfProtection", "false"),
+    ("server.runOnSave", "true"),
+]
+
+
 class StreamlitWatcher:
     def __init__(self):
         self.apps: Dict[str, StreamlitApp] = {}
@@ -120,10 +132,7 @@ class StreamlitWatcher:
         cmd = (
             f"streamlit run {app.filename} "
             + f"--server.port={app.port} "
-            + "--server.headless=true "
-            + "--browser.gatherUsageStats=false "
-            + "--global.dataFrameSerialization=arrow "
-            + "--server.runOnSave=true"
+            + " ".join([f"--{k}={v}" for k, v in _streamlit_args])
         )
         # Set environment and force PYTHONPATH
         current_env = os.environ.copy()
